function [lnQ,omega2,rho,P,f] = wishart_change(X,Y,Lx,Ly)

% Wishart based change detection in two polarimetric SAR images
%
% [lnQ,omega2,rho,P,f] = wishart_change(X,Y,Lx,Ly);
%
% Change detection in polarimetric SAR images
% (multi-look covariance representation)
% by means of the complex Wishart distribution
%
% Perform test for equality of two complex 1x1, 2x2 or 3x3
% variance-covariance matrices representing
%
%   full polarimetry
%   full polarimetry, azimutal symmetry
%   full polarimetry, diagonal case
%   dual polarimetry
%   dual polarimetry, diagonal case
%   single polarization
%
% (to read the data from disk, see freadenvisar)
%
% ShhShh* ShhShv* ShhSvv*     k    a   rho
% ShvShh* ShvShv* ShvSvv*  =  a*   ksi b
% SvvShh* SvvShv* SvvSvv*     rho* b*  zeta
%
% Input
%   X       - covariance matrix input at first time point;
%   Y       - covariance matrix input at second time point
%             X and Y must be 3-D with either
%               9 (full polarimetry),
%               5 (full, azimuthal symmetry),
%               3 (full, diagonal only),
%               4 (dual polarimetry),
%               2 (dual, diagonal only), or
%               1 band(s);
%             reading order is alphabetical in indexes with
%             real parts before imaginary parts (as read by
%             freadenvisar), fx. for full polarimetry:
%               hhhh (real), hhhv (complex), hhvv (complex),
%               hvhv (real), hvvv (complex), vvvv (real)
%   Lx      - number of looks for X
%   Ly      - number of looks for Y (defaults to Lx)
%
% Output
%   lnQ     - logarithm of test quantity Q
%   omega2
%   rho
%   P       - probability of finding smaller Q
%   f       - number of degrees of freedom%
%
% References
% [1] Knut Conradsen, Allan Aasbjerg Nielsen, Jesper Schou, Henning Skriver,
%     "A test statistic in the complex Wishart distribution and its
%     application to change detection in polarimetric SAR data,"
%     IEEE Transactions on Geoscience and Remote Sensing 41(1):4-19, 2003.
%
% [2] Jesper Schou, Henning Skriver, Allan Aasbjerg Nielsen, Knut Conradsen,
%     "CFAR edge detector for polarimetric SAR images,"
%     IEEE Transactions on Geoscience and Remote Sensing 41(1):20-32, 2003.

% (c) Copyright 2012-2018
% Allan Aasbjerg Nielsen
% alan@dtu.dk, people.compute.dtu.dk/alan
% 6 Mar 2012 - 27 Oct 2018

if nargin<1
    help wishart_change
    error('wishart_change: wrong input');
end
if nargout>5
    help wishart_change
    error('wishart_change: wrong output');
end
if nargin<4
    Ly = Lx;
end

if ischar(X), X = freadenvisar(X); end
if ischar(Y), Y = freadenvisar(Y); end

pX = size(X);
pY = size(Y);
if pX~=pY
    error('wishart_change: X and Y must have same dimension');
end
nrows = pX(1);
ncols = pX(2);
if  ndims(X)==3
    nvars = pX(3);
elseif ndims(X)==2
    nvars = 1;
else
    error('wishart_change: wrong dimensionality of input');
end

n = Lx; % #looks time 1
m = Ly; % #looks time 2
N = n+m;
X = n*X;
Y = m*Y;

if nvars==9 % fully polarimetric data
%     fprintf('wishart_change: full polarimetry\n');
    p = 3;
    if nargout>1
        f = p^2;
        rho = 1-(2*f-1)*(1/n+1/m-1/N)/(6*p);
        omega2 = -f/4*(1-1/rho)^2+f*(f-1)*(1/n^2+1/m^2-1/N^2)/(24*rho^2);
    end
elseif nvars==5 % full polarimetry, azimuthal symmetry
%     fprintf('wishart_change: full polarimetry, azimuthal symmetry\n');
    p = 3;
    if nargout>1
        p1 = 2;
        f1 = p1^2;
        rho1 = 1-(2*f1-1)*(1/n+1/m-1/N)/(6*p1);
        p2 = 1;
        f2 = p2^2;
        rho2 = 1-(2*f2-1)*(1/n+1/m-1/N)/(6*p2);
        f = f1 + f2;
        rho = (f1*rho1 + f2*rho2)/f;
        omega2 = -f/4*(1-1/rho)^2 + ...
            (f1*(f1-1)+f2*(f2-1))/24*(1/n^2+1/m^2-1/N^2)/rho^2;
    end
elseif nvars==3 % full polarimetry, diagonal only
%     fprintf('wishart_change: full polarimetry, diagonal only\n');
    p = 3;
    if nargout>1
        p1 = 1;
        f1 = p1^2;
        rho1 = 1-(2*f1-1)*(1/n+1/m-1/N)/(6*p1);
        p2 = 1;
        f2 = p2^2;
        rho2 = 1-(2*f2-1)*(1/n+1/m-1/N)/(6*p2);
        p3 = 1;
        f3 = p3^2;
        rho3 = 1-(2*f3-1)*(1/n+1/m-1/N)/(6*p3);
        f = f1 + f2 + f3;
        rho = (f1*rho1 + f2*rho2 + f3*rho3)/f;
        omega2 = -f/4*(1-1/rho)^2 + ...
            (f1*(f1-1)+f2*(f2-1)+f3*(f3-1))/24*(1/n^2+1/m^2-1/N^2)/rho^2;
    end
elseif nvars==4 % dual polarimetry
%     fprintf('wishart_change: dual polarimetry\n');
    p = 2;
    if nargout>1
        f = p^2;
        rho = 1-(2*f-1)*(1/n+1/m-1/N)/(6*p);
        omega2 = -f/4*(1-1/rho)^2 ...
            + f*(f-1)/24*(1/n^2+1/m^2-1/N^2)/rho^2;
    end
elseif nvars==2 % dual polarimetry, diagonal only
%     fprintf('wishart_change: dual polarimetry, diagonal only\n');
    p = 2;
    if nargout>1
        p1 = 1;
        f1 = p1^2;
        rho1 = 1-(2*f1-1)*(1/n+1/m-1/N)/(6*p1);
        p2 = 1;
        f2 = p2^2;
        rho2 = 1-(2*f2-1)*(1/n+1/m-1/N)/(6*p2);
        f = f1 + f2;
        rho = (f1*rho1 + f2*rho2)/f;
        omega2 = -f/4*(1-1/rho)^2 + ...
            (f1*(f1-1)+f2*(f2-1))/24*(1/n^2+1/m^2-1/N^2)/rho^2;
    end
elseif nvars==1 % one channel data only
%     fprintf('wishart_change: one channel data\n');
    p = 1;
    if nargout>1
        f = p^2;
        rho = 1-(2*f-1)*(1/n+1/m-1/N)/(6*p);
        omega2 = -f/4*(1-1/rho)^2+f*(f-1)/24*(1/n^2+1/m^2-1/N^2)/rho^2;
    end
else
    error('wishart_change: input (9, 5, 3), (4, 2) or 1 band(s), real (and complex)');
end
det1 = wishart_det(X);
det2 = wishart_det(Y);
X = X + Y;
clear Y
det3 = wishart_det(X);
clear X
cst = p*(N*log(N)-n*log(n)-m*log(m));
lnQ = cst+n*log(det1)+m*log(det2)-N*log(det3);

if nargout>3
    clear det1 det2 det3
    P = (1 - omega2)*gammainc(-rho*lnQ,0.5*f  ) ...
           + omega2 *gammainc(-rho*lnQ,0.5*f+4);
end
